@@ -3,16 +3,18 @@
 #include <Rdefines.h>
 #include <stdio.h>
 
-
 void check_is_xptr(SEXP s) {
-    if (TYPEOF(s) != EXTPTRSXP) {
-        error("expect an externalptr");
-    }
+    if (TYPEOF(s) != EXTPTRSXP) Rf_error("expect an externalptr");
 }
 
+void check_is_lgl(SEXP s) {
+    if (!Rf_isLogical(s)) Rf_error("expect a logical value");
+}
+
+
 void* str2ptr(SEXP p) {
-    if (!isString(p)) error("expect a string of pointer address");
-    return (void*) strtol(CHAR(STRING_PTR(p)[0]), NULL, 0);
+    if (!isString(p)) Rf_error("expect a string of pointer address");
+    return (void*) strtol(CHAR(STRING_PTR_RO(p)[0]), NULL, 0);
 }
 
 SEXP new_xptr(SEXP p, SEXP tag, SEXP protected) {
@@ -28,10 +30,15 @@ SEXP is_null_xptr(SEXP s) {
     return Rf_ScalarLogical(R_ExternalPtrAddr(s) == NULL);
 }
 
-SEXP xptr_address(SEXP s) {
+SEXP xptr_address(SEXP s, SEXP f) {
     check_is_xptr(s);
-    char* buf[20];
-    sprintf((char*) buf, "%p", R_ExternalPtrAddr(s));
+    check_is_lgl(f);
+    const int n = 20;
+    char* buf[n];
+    if (Rf_asLogical(f))
+        snprintf((char*) buf, n, "%p", R_ExternalPtrAddr(s));
+    else
+        snprintf((char*) buf, n, "%lld", (long long int) R_ExternalPtrAddr(s));
     return Rf_mkString((char*) buf);
 }
 
@@ -71,8 +78,8 @@ SEXP set_xptr_protected(SEXP s, SEXP protected) {
 
 SEXP register_xptr_finalizer(SEXP s, SEXP f, SEXP onexit) {
     check_is_xptr(s);
-    if (!isFunction(f)) error("expect a function");
-    if (!isNumeric(onexit)) error("expect TRUE/FALSE");
+    if (!isFunction(f)) Rf_error("expect a function");
+    if (!isNumeric(onexit)) Rf_error("expect TRUE/FALSE");
     R_RegisterFinalizerEx(s, f, Rf_asLogical(onexit));
     return R_NilValue;
 }
@@ -82,7 +89,7 @@ static const R_CallMethodDef CallEntries[] = {
     {"new_xptr", (DL_FUNC) &new_xptr, 3},
     {"null_xptr", (DL_FUNC) &null_xptr, 2},
     {"is_null_xptr", (DL_FUNC) &is_null_xptr, 1},
-    {"xptr_address", (DL_FUNC) &xptr_address, 1},
+    {"xptr_address", (DL_FUNC) &xptr_address, 2},
     {"xptr_tag", (DL_FUNC) &xptr_tag, 1},
     {"xptr_protected", (DL_FUNC) &xptr_protected, 1},
     {"xptr_clear", (DL_FUNC) &xptr_clear, 1},
@@ -96,4 +103,5 @@ static const R_CallMethodDef CallEntries[] = {
 void R_init_xptr(DllInfo *dll) {
     R_registerRoutines(dll, NULL, CallEntries, NULL, NULL);
     R_useDynamicSymbols(dll, FALSE);
+    R_forceSymbols(dll, TRUE);
 }
